/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef JAC_AP_LIDARPERCEPTIONSERVICE_LIDARPERCEPTIONSERVICE_PROXY_H
#define JAC_AP_LIDARPERCEPTIONSERVICE_LIDARPERCEPTIONSERVICE_PROXY_H

#include "ara/com/internal/proxy/proxy_adapter.h"
#include "ara/com/internal/proxy/event_adapter.h"
#include "ara/com/internal/proxy/field_adapter.h"
#include "ara/com/internal/proxy/method_adapter.h"
#include "ara/com/crc_verification.h"
#include "jac/ap/lidarperceptionservice/lidarperceptionservice_common.h"
#include <string>

namespace jac {
namespace ap {
namespace lidarperceptionservice {
namespace proxy {
namespace events {
    using LiDARPerception2FusionEvent = ara::com::internal::proxy::event::EventAdapter<::perception::vector_LiDARPerception2Fusion>;
    using RulebasedResultEvent = ara::com::internal::proxy::event::EventAdapter<::PerceptionBoundingBoxArray>;
    static constexpr ara::com::internal::EntityId LiDARPerceptionServiceLiDARPerception2FusionEventId = 50932U; //LiDARPerception2FusionEvent_event_hash
    static constexpr ara::com::internal::EntityId LiDARPerceptionServiceRulebasedResultEventId = 6953U; //RulebasedResultEvent_event_hash
}

namespace fields {
}

namespace methods {

} // namespace methods

class LiDARPerceptionServiceProxy {
private:
    std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> proxyAdapter;
public:
    using HandleType = ara::com::HandleType;
    class ConstructionToken {
    public:
        explicit ConstructionToken(const std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> & proxy)
            : proxyAdapter(proxy),
              LiDARPerception2FusionEvent(proxyAdapter->GetProxy(), events::LiDARPerceptionServiceLiDARPerception2FusionEventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
              RulebasedResultEvent(proxyAdapter->GetProxy(), events::LiDARPerceptionServiceRulebasedResultEventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()){
        }

        ConstructionToken(ConstructionToken&& other) = default;
        ConstructionToken& operator=(ConstructionToken && other) = default;

        ConstructionToken(const ConstructionToken&) = delete;
        ConstructionToken& operator = (const ConstructionToken&) = delete;
        
        virtual ~ConstructionToken() = default;

        /* Do not use such internal interfaces!!! */
        ara::core::Result<void> InitializeAll()
        {
            ara::core::Result<void> initResult;
            return initResult;
        }
        std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> GetProxyAdapter()
        {
            return proxyAdapter;
        }
        events::LiDARPerception2FusionEvent GetLiDARPerception2FusionEvent() noexcept { return std::move(LiDARPerception2FusionEvent); }
        events::RulebasedResultEvent GetRulebasedResultEvent() noexcept { return std::move(RulebasedResultEvent); }
    private:
        std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> proxyAdapter;
        events::LiDARPerception2FusionEvent LiDARPerception2FusionEvent;
        events::RulebasedResultEvent RulebasedResultEvent;
    };

    virtual ~LiDARPerceptionServiceProxy()
    {
        LiDARPerception2FusionEvent.UnsetReceiveHandler();
        LiDARPerception2FusionEvent.Unsubscribe();
        RulebasedResultEvent.UnsetReceiveHandler();
        RulebasedResultEvent.Unsubscribe();

    }

    explicit LiDARPerceptionServiceProxy(const HandleType &handle)
        : proxyAdapter(std::make_shared<ara::com::internal::proxy::ProxyAdapter>(::jac::ap::lidarperceptionservice::LiDARPerceptionService::ServiceIdentifier, handle)),
          LiDARPerception2FusionEvent(proxyAdapter->GetProxy(), events::LiDARPerceptionServiceLiDARPerception2FusionEventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
          RulebasedResultEvent(proxyAdapter->GetProxy(), events::LiDARPerceptionServiceRulebasedResultEventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()){
        }

    void ThrowError(const ara::core::Result<void>& result) const
    {
        if (!(result.HasValue())) {
#ifndef NOT_SUPPORT_EXCEPTIONS
            ara::core::ErrorCode errorcode(result.Error());
            throw ara::com::ComException(std::move(errorcode));
#else
            std::cerr << "Error: Not support exception, create proxy failed!"<< std::endl;
#endif
        }
    }

    LiDARPerceptionServiceProxy(const LiDARPerceptionServiceProxy&) = delete;
    LiDARPerceptionServiceProxy& operator=(const LiDARPerceptionServiceProxy&) = delete;

    LiDARPerceptionServiceProxy(LiDARPerceptionServiceProxy&&) = default;
    LiDARPerceptionServiceProxy& operator=(LiDARPerceptionServiceProxy&&) = default;
    LiDARPerceptionServiceProxy(ConstructionToken&& token) noexcept
        : proxyAdapter(token.GetProxyAdapter()),
          LiDARPerception2FusionEvent(token.GetLiDARPerception2FusionEvent()),
          RulebasedResultEvent(token.GetRulebasedResultEvent()){
        static_cast<void>(std::move(token));
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        const ara::com::HandleType &handle)
    {
        std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> preProxyAdapter =
            std::make_shared<ara::com::internal::proxy::ProxyAdapter>(
               ::jac::ap::lidarperceptionservice::LiDARPerceptionService::ServiceIdentifier, handle);
        ConstructionToken token{std::move(preProxyAdapter)};
        ara::core::Result<void> initResult {token.InitializeAll()};
        if (initResult.HasValue()) {
            return ara::core::Result<ConstructionToken>(std::move(token));
        } else {
            const ara::core::ErrorCode errorcode{initResult.Error()};
            ara::core::Result<LiDARPerceptionServiceProxy::ConstructionToken> preResult{errorcode};
            return preResult;
        }
    }

    static ara::com::FindServiceHandle StartFindService(
        const ara::com::FindServiceHandler<ara::com::internal::proxy::ProxyAdapter::HandleType>& handler,
        const ara::com::InstanceIdentifier instance)
    {
        return ara::com::internal::proxy::ProxyAdapter::StartFindService(handler, ::jac::ap::lidarperceptionservice::LiDARPerceptionService::ServiceIdentifier, instance);
    }

    static ara::com::FindServiceHandle StartFindService(
        const ara::com::FindServiceHandler<ara::com::internal::proxy::ProxyAdapter::HandleType> handler,
        const ara::core::InstanceSpecifier specifier)
    {
        return ara::com::internal::proxy::ProxyAdapter::StartFindService(handler, ::jac::ap::lidarperceptionservice::LiDARPerceptionService::ServiceIdentifier, specifier);
    }

    static ara::com::ServiceHandleContainer<ara::com::internal::proxy::ProxyAdapter::HandleType> FindService(
        const ara::com::InstanceIdentifier instance)
    {
        return ara::com::internal::proxy::ProxyAdapter::FindService(::jac::ap::lidarperceptionservice::LiDARPerceptionService::ServiceIdentifier, instance);
    }

    static ara::com::ServiceHandleContainer<ara::com::internal::proxy::ProxyAdapter::HandleType> FindService(
        const ara::core::InstanceSpecifier specifier)
    {
        return ara::com::internal::proxy::ProxyAdapter::FindService(::jac::ap::lidarperceptionservice::LiDARPerceptionService::ServiceIdentifier, specifier);
    }

    static void StopFindService(const ara::com::FindServiceHandle& handle)
    {
        ara::com::internal::proxy::ProxyAdapter::StopFindService(handle);
    }

    HandleType GetHandle() const
    {
        return proxyAdapter->GetHandle();
    }
    bool SetEventThreadNumber(const std::uint16_t number, const std::uint16_t queueSize)
    {
        return proxyAdapter->SetEventThreadNumber(number, queueSize);
    }
    events::LiDARPerception2FusionEvent LiDARPerception2FusionEvent;
    events::RulebasedResultEvent RulebasedResultEvent;
};
} // namespace proxy
} // namespace lidarperceptionservice
} // namespace ap
} // namespace jac

#endif // JAC_AP_LIDARPERCEPTIONSERVICE_LIDARPERCEPTIONSERVICE_PROXY_H
